<?php

namespace SGalinski\SgMail\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\VersionNumberUtility;
use TYPO3\CMS\Extbase\Object\ObjectManagerInterface;
use TYPO3\CMS\Extbase\Persistence\Generic\Typo3QuerySettings;
use TYPO3\CMS\Extbase\Persistence\Repository;

/**
 * Abstract Repository
 */
abstract class AbstractRepository extends Repository {
	/**
	 * AbstractRepository constructor.
	 *
	 * @param ObjectManagerInterface $objectManager
	 */
	public function __construct(ObjectManagerInterface $objectManager) {
		parent::__construct($objectManager);
		if (version_compare(VersionNumberUtility::getCurrentTypo3Version(), '10.4.0', '<')) {
			/** @var Typo3QuerySettings $querySettings */
			$querySettings = $this->objectManager->get(Typo3QuerySettings::class);
		} else {
			$querySettings = GeneralUtility::makeInstance(Typo3QuerySettings::class);
		}

		$querySettings->setRespectStoragePage(FALSE);
		if (TYPO3_MODE === 'BE') {
			$querySettings->setIgnoreEnableFields(TRUE);
		}

		$this->setDefaultQuerySettings($querySettings);
	}

	/**
	 * Persist changes to database
	 */
	public function persist() {
		$this->persistenceManager->persistAll();
	}
}
